import { forwardRef, EventEmitter, Input, Output, Component, ElementRef, ContentChildren, Optional, NgModule } from '@angular/core';
import * as GC from '@grapecity/inputman';
import { __decorate, __metadata } from 'tslib';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';

const IMCtrl = "__imctrl";
function Property(setter, getter, isPramArr, supportWithinConfig, order) {
    var saveToTargetProDic = function (target, propertyName) {
        var methods = {
            setter: setter,
            getter: getter,
            isPramArr: isPramArr,
            supportWithinConfig: (typeof supportWithinConfig === "boolean") ? supportWithinConfig : true,
            order: (typeof order === "number") ? order : -1
        };
        if (!target.propDic) {
            target.propDic = {};
        }
        target.propDic[propertyName] = methods;
    };
    return saveToTargetProDic;
}
const isEmpty = function (value) {
    return value === undefined || value === null || value === "";
};
const getMethodPrefix = function (type) {
    if (type === MethodType.getter) {
        return "get";
    }
    else if (type === MethodType.setter) {
        return "set";
    }
};
var MethodType;
(function (MethodType) {
    MethodType[MethodType["getter"] = 0] = "getter";
    MethodType[MethodType["setter"] = 1] = "setter";
})(MethodType || (MethodType = {}));

function provideParent(component) {
    return { provide: GcComponents, useExisting: forwardRef(() => component) };
}
class GcComponents {
    constructor(ref) {
        this.onInitialized = new EventEmitter();
        this._container = ref.nativeElement;
    }
    ngOnChanges(changes) {
        if (!this._imCtrl) {
            return;
        }
        Object.keys(changes).forEach((key) => {
            var currentValue = changes[key].currentValue;
            this.setValueToIMControl(key, currentValue);
        });
    }
    setValueToIMControl(key, value) {
        if (value === undefined) {
            return;
        }
        if (this.isSetAccessor(key)) {
            this._imCtrl[key] = value;
            return;
        }
        var setter = this.tryGetPropertyOperator(key, MethodType.setter);
        if (this.hasRealMethod(setter)) {
            if (this.propDic && this.propDic[key] && this.propDic[key].isPramArr === true) {
                this._imCtrl[setter].apply(this._imCtrl, value);
            }
            else {
                this._imCtrl[setter](value);
            }
        }
    }
    isSetAccessor(key) {
        let descriptor = Object.getOwnPropertyDescriptor(this._imCtrl.constructor.prototype, key);
        if (descriptor && descriptor.set) {
            return true;
        }
        return false;
    }
    ngAfterContentInit() {
        this.initGcComponent();
        if (typeof this._imCtrl === 'object') {
            this.onInitialized.emit(this._imCtrl);
        }
        this.bindEvent();
        if (this.propDic) {
            var unSupportWithinConfigProps = Object.keys(this.propDic).filter(p => this.propDic[p] && this.propDic[p].supportWithinConfig === false);
            unSupportWithinConfigProps.sort((p1, p2) => {
                return this.propDic[p1].order - this.propDic[p2].order;
            });
            unSupportWithinConfigProps.forEach(p => {
                this.setValueToIMControl(p, this[p]);
            });
        }
        this._container[IMCtrl] = this._imCtrl;
    }
    tryGetPropertyOperator(propertyName, methodType) {
        var method = "";
        if (this.propDic && this.propDic[propertyName]) {
            method = methodType === MethodType.getter ? this.propDic[propertyName].getter : this.propDic[propertyName].setter;
            if (!isEmpty(method)) {
                return method;
            }
        }
        return getMethodPrefix(methodType) + propertyName[0].toUpperCase() + propertyName.slice(1);
    }
    hasRealMethod(methodName) {
        return this._imCtrl && typeof this._imCtrl[methodName] === "function";
    }
    getNestedIMControl() {
        return this._imCtrl;
    }
    createTemplate(element) {
        this._container.appendChild(element);
        if (element && this.name) {
            element.setAttribute('name', this.name);
        }
    }
}
GcComponents.propDecorators = {
    name: [{ type: Input }],
    onInitialized: [{ type: Output }]
};

class GcCalculatorComponent extends GcComponents {
    constructor(ref) {
        super(ref);
    }
    initGcComponent() {
        var element = document.createElement('div');
        this.createTemplate(element);
        this._imCtrl = new GC.InputMan.GcCalculator(element, this);
    }
    bindEvent() { }
}
GcCalculatorComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-calculator',
                template: ``
            },] }
];
GcCalculatorComponent.ctorParameters = () => [
    { type: ElementRef }
];
GcCalculatorComponent.propDecorators = {
    value: [{ type: Input }],
    showOutput: [{ type: Input }],
    visible: [{ type: Input }],
    enabled: [{ type: Input }],
    buttonText: [{ type: Input }]
};

class GcCalendarComponent extends GcComponents {
    constructor(ref) {
        super(ref);
        this.onClickDate = new EventEmitter();
        this.onScrolled = new EventEmitter();
        this.onFocusDateChanged = new EventEmitter();
        this.onSelectedDateChanged = new EventEmitter();
        // For V2.1 Compatibility, we have to keep this property or event
        this.focusDateChange = new EventEmitter();
        this.selectedDateChange = new EventEmitter();
    }
    initGcComponent() {
        var element = document.createElement('div');
        this.createTemplate(element);
        if (Array.isArray(this.calendarDimensions)) {
            this.calendarDimensions = {
                width: this.calendarDimensions[0],
                height: this.calendarDimensions[1]
            };
        }
        this._imCtrl = new GC.InputMan.GcCalendar(element, this);
    }
    bindEvent() {
        if (!this._imCtrl) {
            return;
        }
        this._imCtrl.onClickDate((s) => { this.onClickDate.emit(s); });
        this._imCtrl.onScrolled((s) => { this.onScrolled.emit(s); });
        this._imCtrl.onFocusDateChanged((s) => {
            this.onFocusDateChanged.emit(s);
            this.focusDateChange.emit(s.getFocusDate());
        });
        this._imCtrl.onSelectedDateChanged((s) => {
            this.onSelectedDateChanged.emit(s);
            this.selectedDateChange.emit(s.getSelectedDate());
        });
    }
}
GcCalendarComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-calendar',
                template: ``
            },] }
];
GcCalendarComponent.ctorParameters = () => [
    { type: ElementRef }
];
GcCalendarComponent.propDecorators = {
    visible: [{ type: Input }],
    maxSelectionCount: [{ type: Input }],
    selectionMode: [{ type: Input }],
    selectedDate: [{ type: Input }],
    enabled: [{ type: Input }],
    weekTitleSelect: [{ type: Input }],
    weekNumberSelect: [{ type: Input }],
    focusDate: [{ type: Input }],
    maxDate: [{ type: Input }],
    minDate: [{ type: Input }],
    allowSelection: [{ type: Input }],
    emptyRows: [{ type: Input }],
    firstDayOfWeek: [{ type: Input }],
    firstFiscalMonth: [{ type: Input }],
    firstMonthInView: [{ type: Input }],
    yearMonthFormat: [{ type: Input }],
    headerFormat: [{ type: Input }],
    showZoomButton: [{ type: Input }],
    showRokuyou: [{ type: Input }],
    showHeader: [{ type: Input }],
    showToday: [{ type: Input }],
    showWeekNumber: [{ type: Input }],
    showTrailing: [{ type: Input }],
    calendarYear: [{ type: Input }],
    showNavigator: [{ type: Input }],
    navigatorOrientation: [{ type: Input }],
    overrideTipText: [{ type: Input }],
    calendarType: [{ type: Input }],
    calendarDimensions: [{ type: Input }],
    scrollRate: [{ type: Input }],
    onClickDate: [{ type: Output }],
    onScrolled: [{ type: Output }],
    onFocusDateChanged: [{ type: Output }],
    onSelectedDateChanged: [{ type: Output }],
    focusDateChange: [{ type: Output }],
    selectedDateChange: [{ type: Output }]
};
__decorate([
    Property("setCalendarDimensions", "", true),
    __metadata("design:type", Array)
], GcCalendarComponent.prototype, "calendarDimensions", void 0);

class GcComboBoxColumnComponent {
}
GcComboBoxColumnComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-combo-box-column',
                template: ``
            },] }
];
GcComboBoxColumnComponent.propDecorators = {
    name: [{ type: Input }],
    label: [{ type: Input }],
    width: [{ type: Input }],
    isValuePath: [{ type: Input }],
    visible: [{ type: Input }],
    clickSort: [{ type: Input }],
    contentAlignment: [{ type: Input }],
    headerAlignment: [{ type: Input }],
    showTip: [{ type: Input }]
};
class GcComboBoxComponent extends GcComponents {
    constructor(ref) {
        super(ref);
        this.selectedChanged = new EventEmitter();
        this.checkedChanged = new EventEmitter();
        this.dropDownClosed = new EventEmitter();
        this.dropDownOpened = new EventEmitter();
        this.textChanged = new EventEmitter();
        this.spinDown = new EventEmitter();
        this.spinUp = new EventEmitter();
        this.itemsChanged = new EventEmitter();
        this.itemsChange = new EventEmitter();
        this.selectedValueChange = new EventEmitter();
        this.selectedIndexChange = new EventEmitter();
        this.checkedValuesChange = new EventEmitter();
        this.selectedItemChange = new EventEmitter();
        this.checkedItemsChange = new EventEmitter();
        this.checkedIndicesChange = new EventEmitter();
    }
    initGcComponent() {
        if (this.columnComponents && this.columnComponents.length > 0) {
            this.columns = [];
            this.columnComponents.forEach((column) => {
                this.columns.push({
                    name: column.name,
                    label: column.label,
                    width: column.width,
                    isValuePath: column.isValuePath,
                    visible: column.visible,
                    clickSort: column.clickSort,
                    contentAlignment: column.contentAlignment,
                    headerAlignment: column.headerAlignment,
                    showTip: column.showTip
                });
            });
        }
        var element = document.createElement('select');
        this.createTemplate(element);
        this._imCtrl = new GC.InputMan.GcComboBox(element, this);
    }
    bindEvent() {
        if (!this._imCtrl) {
            return;
        }
        this._imCtrl.addEventListener(GC.InputMan.GcComboBoxEvent.SelectedChanged, (eventArgs) => {
            this.selectedChanged.emit(eventArgs);
            this.selectedValueChange.emit(this._imCtrl.getSelectedValue());
            this.selectedIndexChange.emit(this._imCtrl.getSelectedIndex());
            this.selectedItemChange.emit(this._imCtrl.getSelectedItem());
        });
        this._imCtrl.addEventListener(GC.InputMan.GcComboBoxEvent.CheckedChanged, (eventArgs) => {
            this.checkedChanged.emit(eventArgs);
            this.checkedValuesChange.emit(this._imCtrl.getCheckedValues());
            this.checkedItemsChange.emit(this._imCtrl.getCheckedItems());
            this.checkedIndicesChange.emit(this._imCtrl.getCheckedIndices());
        });
        this._imCtrl.addEventListener(GC.InputMan.GcComboBoxEvent.DropDownClosed, (eventArgs) => {
            this.dropDownClosed.emit(eventArgs);
        });
        this._imCtrl.addEventListener(GC.InputMan.GcComboBoxEvent.DropDownOpened, (eventArgs) => {
            this.dropDownOpened.emit(eventArgs);
        });
        this._imCtrl.addEventListener(GC.InputMan.GcComboBoxEvent.TextChanged, (eventArgs) => {
            this.textChanged.emit(eventArgs);
        });
        this._imCtrl.addEventListener(GC.InputMan.GcComboBoxEvent.SpinDown, (eventArgs) => {
            this.spinDown.emit(eventArgs);
        });
        this._imCtrl.addEventListener(GC.InputMan.GcComboBoxEvent.SpinUp, (eventArgs) => {
            this.spinUp.emit(eventArgs);
        });
        this._imCtrl.addEventListener(GC.InputMan.GcComboBoxEvent.ItemsChanged, (eventArgs) => {
            this.itemsChanged.emit(eventArgs);
            this.itemsChange.emit(this._imCtrl.getItems());
        });
    }
}
GcComboBoxComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-combo-box',
                template: ``,
                providers: [provideParent(GcComboBoxComponent)]
            },] }
];
GcComboBoxComponent.ctorParameters = () => [
    { type: ElementRef }
];
GcComboBoxComponent.propDecorators = {
    items: [{ type: Input }],
    selectedValue: [{ type: Input }],
    allowDropDownResize: [{ type: Input }],
    readOnly: [{ type: Input }],
    autoSelect: [{ type: Input }],
    selectedIndex: [{ type: Input }],
    autoFilter: [{ type: Input }],
    visible: [{ type: Input }],
    enabled: [{ type: Input }],
    showTip: [{ type: Input }],
    isEditable: [{ type: Input }],
    watermarkDisplayNullText: [{ type: Input }],
    watermarkNullText: [{ type: Input }],
    highlightText: [{ type: Input }],
    exitOnEnterKey: [{ type: Input }],
    useClipboard: [{ type: Input }],
    acceptsCrlf: [{ type: Input }],
    ellipsis: [{ type: Input }],
    ellipsisString: [{ type: Input }],
    exitOnLeftRightKey: [{ type: Input }],
    tabIndex: [{ type: Input }],
    editMode: [{ type: Input }],
    checkedValues: [{ type: Input }],
    selectedItem: [{ type: Input }],
    checkedItems: [{ type: Input }],
    checkedIndices: [{ type: Input }],
    showDropDownButton: [{ type: Input }],
    dropDownButtonPosition: [{ type: Input }],
    showSpinButton: [{ type: Input }],
    spinButtonPosition: [{ type: Input }],
    dropDownWidth: [{ type: Input }],
    dropDownHeight: [{ type: Input }],
    minPrefixLength: [{ type: Input }],
    isMultiSelect: [{ type: Input }],
    dropDownOverflow: [{ type: Input }],
    visibleItems: [{ type: Input }],
    itemHeight: [{ type: Input }],
    columns: [{ type: Input }],
    showHeader: [{ type: Input }],
    valueMemberPath: [{ type: Input }],
    displayMemberPath: [{ type: Input }],
    checkOnClick: [{ type: Input }],
    itemTemplate: [{ type: Input }],
    headerTemplate: [{ type: Input }],
    footerTemplate: [{ type: Input }],
    spinWheel: [{ type: Input }],
    emptyTemplate: [{ type: Input }],
    load: [{ type: Input }],
    pageSize: [{ type: Input }],
    virtualMode: [{ type: Input }],
    generatingItem: [{ type: Input }],
    selectTemplate: [{ type: Input }],
    formatItem: [{ type: Input }],
    allowColumnResize: [{ type: Input }],
    colHeaderHeight: [{ type: Input }],
    container: [{ type: Input }],
    autoDropDown: [{ type: Input }],
    dropDownButtonVisible: [{ type: Input }],
    showClearButton: [{ type: Input }],
    autoScale: [{ type: Input }],
    minScaleFactor: [{ type: Input }],
    multipleItemSeparator: [{ type: Input }],
    selectedChanged: [{ type: Output }],
    checkedChanged: [{ type: Output }],
    dropDownClosed: [{ type: Output }],
    dropDownOpened: [{ type: Output }],
    textChanged: [{ type: Output }],
    spinDown: [{ type: Output }],
    spinUp: [{ type: Output }],
    itemsChanged: [{ type: Output }],
    itemsChange: [{ type: Output }],
    selectedValueChange: [{ type: Output }],
    selectedIndexChange: [{ type: Output }],
    checkedValuesChange: [{ type: Output }],
    selectedItemChange: [{ type: Output }],
    checkedItemsChange: [{ type: Output }],
    checkedIndicesChange: [{ type: Output }],
    columnComponents: [{ type: ContentChildren, args: [GcComboBoxColumnComponent,] }]
};
__decorate([
    Property("", "", false, false),
    __metadata("design:type", Object)
], GcComboBoxComponent.prototype, "selectedValue", void 0);
__decorate([
    Property("", "", false, false),
    __metadata("design:type", Number)
], GcComboBoxComponent.prototype, "selectedIndex", void 0);
__decorate([
    Property("setEditable"),
    __metadata("design:type", Boolean)
], GcComboBoxComponent.prototype, "isEditable", void 0);
__decorate([
    Property("", "", false, false),
    __metadata("design:type", Object)
], GcComboBoxComponent.prototype, "checkedValues", void 0);
__decorate([
    Property("", "", false, false),
    __metadata("design:type", Object)
], GcComboBoxComponent.prototype, "selectedItem", void 0);
__decorate([
    Property("", "", false, false),
    __metadata("design:type", Object)
], GcComboBoxComponent.prototype, "checkedItems", void 0);
__decorate([
    Property("", "", false, false),
    __metadata("design:type", Array)
], GcComboBoxComponent.prototype, "checkedIndices", void 0);

class GcInputManBaseComponent extends GcComponents {
    constructor(ref) {
        super(ref);
        this.onEditStatusChanged = new EventEmitter();
        this.onInvalidInput = new EventEmitter();
        this.onKeyExit = new EventEmitter();
        this.onTextChanged = new EventEmitter();
        this.onInput = new EventEmitter();
        this.onFocusOut = new EventEmitter();
        this.onKeyDown = new EventEmitter();
        this.onKeyUp = new EventEmitter();
        this.onSyncValueToOriginalInput = new EventEmitter();
        // For V2.1 Compatibility, we have to keep this property or event
        this.textChange = new EventEmitter();
    }
    bindEvent() {
        if (!this._imCtrl) {
            return;
        }
        this._imCtrl.onEditStatusChanged((s) => { this.onEditStatusChanged.emit(s); });
        this._imCtrl.onInvalidInput((s) => { this.onInvalidInput.emit(s); });
        this._imCtrl.onKeyExit((s) => { this.onKeyExit.emit(s); });
        this._imCtrl.onTextChanged((s) => { this.onTextChanged.emit(s); this.textChange.emit(s.getText()); });
        this._imCtrl.onInput((s) => { this.onInput.emit(s); });
        this._imCtrl.onFocusOut((s) => { this.onFocusOut.emit({ sender: s }); });
        this._imCtrl.onKeyDown((s) => { this.onKeyDown.emit({ sender: s }); });
        this._imCtrl.onKeyUp((s) => { this.onKeyUp.emit({ sender: s }); });
        this._imCtrl.onSyncValueToOriginalInput((v, e) => { this.onSyncValueToOriginalInput.emit({ value: v, element: e }); });
    }
}
GcInputManBaseComponent.propDecorators = {
    visible: [{ type: Input }],
    enabled: [{ type: Input }],
    readOnly: [{ type: Input }],
    exitOnLeftRightKey: [{ type: Input }],
    editMode: [{ type: Input }],
    watermarkDisplayNullText: [{ type: Input }],
    watermarkNullText: [{ type: Input }],
    acceptsCrlf: [{ type: Input }],
    useClipboard: [{ type: Input }],
    exitOnEnterKey: [{ type: Input }],
    container: [{ type: Input }],
    autoDropDown: [{ type: Input }],
    dropDownButtonVisible: [{ type: Input }],
    autoScale: [{ type: Input }],
    minScaleFactor: [{ type: Input }],
    tabIndex: [{ type: Input }],
    onEditStatusChanged: [{ type: Output }],
    onInvalidInput: [{ type: Output }],
    onKeyExit: [{ type: Output }],
    onTextChanged: [{ type: Output }],
    onInput: [{ type: Output }],
    onFocusOut: [{ type: Output }],
    onKeyDown: [{ type: Output }],
    onKeyUp: [{ type: Output }],
    onSyncValueToOriginalInput: [{ type: Output }],
    textChange: [{ type: Output }]
};

class GcDateTimeComponent extends GcInputManBaseComponent {
    constructor(ref) {
        super(ref);
        this.onSpinDown = new EventEmitter();
        this.onSpinUp = new EventEmitter();
        this.onInvalidRange = new EventEmitter();
        this.onInvalidValue = new EventEmitter();
        this.onNumberChanged = new EventEmitter();
        this.onValueChanged = new EventEmitter();
        // For V2.1 Compatibility, we have to keep this property or event
        this.numberChange = new EventEmitter();
        this.valueChange = new EventEmitter();
    }
    initGcComponent() {
        var element = document.createElement('input');
        this.createTemplate(element);
        this._imCtrl = new GC.InputMan.GcDateTime(element, this);
    }
    bindEvent() {
        if (!this._imCtrl) {
            return;
        }
        super.bindEvent();
        this._imCtrl.onSpinDown((s) => { this.onSpinDown.emit(s); });
        this._imCtrl.onSpinUp((s) => { this.onSpinUp.emit(s); });
        this._imCtrl.onInvalidRange((s) => { this.onInvalidRange.emit(s); });
        this._imCtrl.onInvalidValue((s) => { this.onInvalidValue.emit(s); });
        this._imCtrl.onNumberChanged((s) => {
            this.onNumberChanged.emit(s);
            this.numberChange.emit(s.getNumber());
        });
        this._imCtrl.onValueChanged((s) => {
            this.onValueChanged.emit(s);
            this.valueChange.emit(s.getValue());
        });
    }
}
GcDateTimeComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-datetime',
                template: ``,
                providers: [provideParent(GcDateTimeComponent)]
            },] }
];
GcDateTimeComponent.ctorParameters = () => [
    { type: ElementRef }
];
GcDateTimeComponent.propDecorators = {
    text: [{ type: Input }],
    number: [{ type: Input }],
    maxDate: [{ type: Input }],
    minDate: [{ type: Input }],
    formatPattern: [{ type: Input }],
    spinIncrement: [{ type: Input }],
    spinOnKeys: [{ type: Input }],
    spinWrap: [{ type: Input }],
    allowSpin: [{ type: Input }],
    spinWheel: [{ type: Input }],
    displayFormatPattern: [{ type: Input }],
    watermarkEmptyEraText: [{ type: Input }],
    promptChar: [{ type: Input }],
    AMDesignator: [{ type: Input }],
    PMDesignator: [{ type: Input }],
    twoDigitYearMax: [{ type: Input }],
    hour12Mode: [{ type: Input }],
    tabAction: [{ type: Input }],
    maxMinBehavior: [{ type: Input }],
    midnightAs24: [{ type: Input }],
    showLiterals: [{ type: Input }],
    adjustValueOnFocus: [{ type: Input }],
    useTwoDigitYearMax: [{ type: Input }],
    validateMode: [{ type: Input }],
    clipContent: [{ type: Input }],
    value: [{ type: Input }],
    watermarkDisplayEmptyEraText: [{ type: Input }],
    dropDownConfig: [{ type: Input }],
    exitOnLastChar: [{ type: Input }],
    highlightText: [{ type: Input }],
    showSpinButton: [{ type: Input }],
    spinButtonPosition: [{ type: Input }],
    showDropDownButton: [{ type: Input }],
    dropDownButtonAlignment: [{ type: Input }],
    showClearButton: [{ type: Input }],
    onSpinDown: [{ type: Output }],
    onSpinUp: [{ type: Output }],
    onInvalidRange: [{ type: Output }],
    onInvalidValue: [{ type: Output }],
    onNumberChanged: [{ type: Output }],
    onValueChanged: [{ type: Output }],
    numberChange: [{ type: Output }],
    valueChange: [{ type: Output }]
};
__decorate([
    Property("", "", false, false),
    __metadata("design:type", Number)
], GcDateTimeComponent.prototype, "number", void 0);

class GcDateTimePickerComponent extends GcComponents {
    constructor(ref) {
        super(ref);
        this.selectedDateChanged = new EventEmitter();
        this.selectedValueChange = new EventEmitter();
    }
    initGcComponent() {
        var element = document.createElement('div');
        this.createTemplate(element);
        this._imCtrl = new GC.InputMan.GcDateTimePicker(element, this);
    }
    bindEvent() {
        if (!this._imCtrl) {
            return;
        }
        this._imCtrl.addEventListener(GC.InputMan.GcDateTimePickerEvent.SelectedDateChanged, (sender, eventArgs) => {
            this.selectedDateChanged.emit({ sender: sender, eArgs: eventArgs });
            this.selectedValueChange.emit(this._imCtrl.getValue());
        });
    }
}
GcDateTimePickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-datetime-picker',
                template: ``
            },] }
];
GcDateTimePickerComponent.ctorParameters = () => [
    { type: ElementRef }
];
GcDateTimePickerComponent.propDecorators = {
    selectedValue: [{ type: Input }],
    maxDate: [{ type: Input }],
    minDate: [{ type: Input }],
    type: [{ type: Input }],
    minuteInterval: [{ type: Input }],
    secondsInterval: [{ type: Input }],
    visible: [{ type: Input }],
    hasSeconds: [{ type: Input }],
    showMidnightAs24: [{ type: Input }],
    yearFormat: [{ type: Input }],
    monthFormat: [{ type: Input }],
    dayFormat: [{ type: Input }],
    selectedDateChanged: [{ type: Output }],
    selectedValueChange: [{ type: Output }]
};
__decorate([
    Property("setValue"),
    __metadata("design:type", Date)
], GcDateTimePickerComponent.prototype, "selectedValue", void 0);
__decorate([
    Property('setPickerType'),
    __metadata("design:type", String)
], GcDateTimePickerComponent.prototype, "type", void 0);
__decorate([
    Property('showSeconds', "", false, false),
    __metadata("design:type", String)
], GcDateTimePickerComponent.prototype, "hasSeconds", void 0);
__decorate([
    Property('showMidnightAs24', "", false, false),
    __metadata("design:type", String)
], GcDateTimePickerComponent.prototype, "showMidnightAs24", void 0);

class GcFormPersistenceComponent extends GcComponents {
    constructor(ref) {
        super(ref);
    }
    initGcComponent() {
        var form = this._container.querySelector('form');
        if (form) {
            this._imCtrl = new GC.InputMan.GcFormPersistence(form, this);
            this._imCtrl.persist();
        }
    }
    bindEvent() { }
}
GcFormPersistenceComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-form-persistence',
                template: `<ng-content></ng-content>`,
                providers: [provideParent(GcFormPersistenceComponent)]
            },] }
];
GcFormPersistenceComponent.ctorParameters = () => [
    { type: ElementRef }
];
GcFormPersistenceComponent.propDecorators = {
    id: [{ type: Input }],
    storageMode: [{ type: Input }],
    saveMode: [{ type: Input }]
};

class GcFunctionKeyInfoComponent {
}
GcFunctionKeyInfoComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-function-key-info',
                template: ``
            },] }
];
GcFunctionKeyInfoComponent.propDecorators = {
    key: [{ type: Input }],
    description: [{ type: Input }],
    tipText: [{ type: Input }],
    displayStyle: [{ type: Input }],
    imagePosition: [{ type: Input }]
};
class GcFunctionKeyComponent extends GcComponents {
    constructor(ref) {
        super(ref);
        this.onActived = new EventEmitter();
    }
    initGcComponent() {
        if (this.functionKeyComponents && this.functionKeyComponents.length > 0) {
            this.functionKeys = [];
            this.functionKeyComponents.forEach((functionKeyInfo) => {
                this.functionKeys.push({
                    key: functionKeyInfo.key,
                    description: functionKeyInfo.description,
                    tipText: functionKeyInfo.tipText,
                    displayStyle: functionKeyInfo.displayStyle,
                    imagePosition: functionKeyInfo.imagePosition
                });
            });
        }
        var element = document.createElement('div');
        this.createTemplate(element);
        this._imCtrl = new GC.InputMan.GcFunctionKey(element, this);
    }
    bindEvent() {
        if (!this._imCtrl)
            return;
        this._imCtrl.onActived((s, e) => {
            this.onActived.emit(e);
        });
    }
}
GcFunctionKeyComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-function-key',
                template: ``
            },] }
];
GcFunctionKeyComponent.ctorParameters = () => [
    { type: ElementRef }
];
GcFunctionKeyComponent.propDecorators = {
    keyRepeat: [{ type: Input }],
    handleMouse: [{ type: Input }],
    showTip: [{ type: Input }],
    combinationKeyDisplayMode: [{ type: Input }],
    functionKeys: [{ type: Input }],
    enabled: [{ type: Input }],
    visible: [{ type: Input }],
    functionKeyComponents: [{ type: ContentChildren, args: [GcFunctionKeyInfoComponent,] }],
    onActived: [{ type: Output }]
};
__decorate([
    Property("addFunctionKey", "", true),
    __metadata("design:type", Array)
], GcFunctionKeyComponent.prototype, "functionKeys", void 0);

class GcIconNotifierComponent extends GcComponents {
    constructor(ref, _parent) {
        super(ref);
        this._parent = _parent;
    }
    ngAfterViewInit() {
        if (!this._parent || typeof this._parent !== 'object' ||
            typeof this._parent.getNestedIMControl() !== 'object') {
            return;
        }
        this._imCtrl = new GC.InputMan.GcIconNotifier(this._parent.getNestedIMControl(), this);
        this.onInitialized.emit(this._imCtrl);
    }
    bindEvent() { }
    initGcComponent() {
    }
    onFail() {
        return this._imCtrl.onFail();
    }
    onSuccess() {
        return this._imCtrl.onSuccess();
    }
    onClear() {
        return this._imCtrl.onClear();
    }
}
GcIconNotifierComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-icon-notifier',
                template: ``
            },] }
];
GcIconNotifierComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: GcComponents, decorators: [{ type: Optional }] }
];
GcIconNotifierComponent.propDecorators = {
    direction: [{ type: Input }],
    failIcon: [{ type: Input }],
    successIcon: [{ type: Input }],
    failMessage: [{ type: Input }],
    successMessage: [{ type: Input }]
};
__decorate([
    Property('setFailIconSrc'),
    __metadata("design:type", String)
], GcIconNotifierComponent.prototype, "failIcon", void 0);
__decorate([
    Property('setSuccessIconSrc'),
    __metadata("design:type", String)
], GcIconNotifierComponent.prototype, "successIcon", void 0);

class GcListBoxColumnComponent {
}
GcListBoxColumnComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-list-box-column',
                template: ``
            },] }
];
GcListBoxColumnComponent.propDecorators = {
    name: [{ type: Input }],
    label: [{ type: Input }],
    width: [{ type: Input }],
    isValuePath: [{ type: Input }],
    visible: [{ type: Input }],
    clickSort: [{ type: Input }],
    contentAlignment: [{ type: Input }],
    headerAlignment: [{ type: Input }],
    showTip: [{ type: Input }]
};
class GcListBoxComponent extends GcComponents {
    constructor(ref) {
        super(ref);
        this.checkedChanged = new EventEmitter();
        this.selectedChanged = new EventEmitter();
        this.focusedChanged = new EventEmitter();
        this.itemClick = new EventEmitter();
        this.loadComplete = new EventEmitter();
        this.itemsChanged = new EventEmitter();
        this.selectedIndexChange = new EventEmitter();
        this.checkedIndexChange = new EventEmitter();
        this.selectedIndicesChange = new EventEmitter();
        this.checkedIndicesChange = new EventEmitter();
    }
    initGcComponent() {
        if (this.columnComponents && this.columnComponents.length > 0) {
            this.columns = [];
            this.columnComponents.forEach((column) => {
                this.columns.push({
                    name: column.name,
                    label: column.label,
                    width: column.width,
                    isValuePath: column.isValuePath,
                    visible: column.visible,
                    clickSort: column.clickSort,
                    contentAlignment: column.contentAlignment,
                    headerAlignment: column.headerAlignment,
                    showTip: column.showTip
                });
            });
        }
        var element = document.createElement('select');
        this.createTemplate(element);
        this._imCtrl = new GC.InputMan.GcListBox(element, this);
    }
    bindEvent() {
        if (!this._imCtrl) {
            return;
        }
        this._imCtrl.addEventListener(GC.InputMan.GcListBoxEvent.CheckedChanged, (eventArgs) => {
            this.checkedChanged.emit(eventArgs);
            this.checkedIndexChange.emit(this._imCtrl.getCheckedIndices()[0]);
            this.checkedIndicesChange.emit(this._imCtrl.getCheckedIndices());
        });
        this._imCtrl.addEventListener(GC.InputMan.GcListBoxEvent.SelectedChanged, (eventArgs) => {
            this.selectedChanged.emit(eventArgs);
            this.selectedIndexChange.emit(this._imCtrl.getSelectedIndex());
            this.selectedIndicesChange.emit(this._imCtrl.getSelectedIndices());
        });
        this._imCtrl.addEventListener(GC.InputMan.GcListBoxEvent.FocusedChanged, (eventArgs) => {
            this.focusedChanged.emit(eventArgs);
        });
        this._imCtrl.addEventListener(GC.InputMan.GcListBoxEvent.ItemClick, (sender, eventArgs) => {
            this.itemClick.emit({ sender: sender, eArgs: eventArgs });
        });
        this._imCtrl.addEventListener(GC.InputMan.GcListBoxEvent.LoadComplete, (eventArgs) => {
            this.loadComplete.emit(eventArgs);
        });
        this._imCtrl.addEventListener(GC.InputMan.GcListBoxEvent.ItemsChanged, (eventArgs) => {
            this.itemsChanged.emit(eventArgs);
        });
    }
}
GcListBoxComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-list-box',
                template: ``
            },] }
];
GcListBoxComponent.ctorParameters = () => [
    { type: ElementRef }
];
GcListBoxComponent.propDecorators = {
    items: [{ type: Input }],
    columns: [{ type: Input }],
    virtualMode: [{ type: Input }],
    allowResize: [{ type: Input }],
    selectionMode: [{ type: Input }],
    selectedIndex: [{ type: Input }],
    checkedIndex: [{ type: Input }],
    selectedIndices: [{ type: Input }],
    checkedIndices: [{ type: Input }],
    allowColumnResize: [{ type: Input }],
    overflow: [{ type: Input }],
    width: [{ type: Input }],
    height: [{ type: Input }],
    itemHeight: [{ type: Input }],
    checkOnClick: [{ type: Input }],
    visibleItems: [{ type: Input }],
    visible: [{ type: Input }],
    enabled: [{ type: Input }],
    showTip: [{ type: Input }],
    multiColumn: [{ type: Input }],
    colHeaderHeight: [{ type: Input }],
    showHeader: [{ type: Input }],
    valueMemberPath: [{ type: Input }],
    displayMemberPath: [{ type: Input }],
    showCheckBox: [{ type: Input }],
    itemTemplate: [{ type: Input }],
    headerTemplate: [{ type: Input }],
    footerTemplate: [{ type: Input }],
    emptyTemplate: [{ type: Input }],
    load: [{ type: Input }],
    pageSize: [{ type: Input }],
    generatingItem: [{ type: Input }],
    selectTemplate: [{ type: Input }],
    formatItem: [{ type: Input }],
    checkedChanged: [{ type: Output }],
    selectedChanged: [{ type: Output }],
    focusedChanged: [{ type: Output }],
    itemClick: [{ type: Output }],
    loadComplete: [{ type: Output }],
    itemsChanged: [{ type: Output }],
    selectedIndexChange: [{ type: Output }],
    checkedIndexChange: [{ type: Output }],
    selectedIndicesChange: [{ type: Output }],
    checkedIndicesChange: [{ type: Output }],
    columnComponents: [{ type: ContentChildren, args: [GcListBoxColumnComponent,] }]
};
__decorate([
    Property("", "", false, false),
    __metadata("design:type", Number)
], GcListBoxComponent.prototype, "selectedIndex", void 0);
__decorate([
    Property("", "", false, false),
    __metadata("design:type", Number)
], GcListBoxComponent.prototype, "checkedIndex", void 0);
__decorate([
    Property("", "", false, false),
    __metadata("design:type", Array)
], GcListBoxComponent.prototype, "selectedIndices", void 0);
__decorate([
    Property("", "", false, false),
    __metadata("design:type", Array)
], GcListBoxComponent.prototype, "checkedIndices", void 0);
__decorate([
    Property("resizeWidth"),
    __metadata("design:type", Object)
], GcListBoxComponent.prototype, "width", void 0);
__decorate([
    Property("resizeHeight"),
    __metadata("design:type", Number)
], GcListBoxComponent.prototype, "height", void 0);
__decorate([
    Property('setVisibleItemCount'),
    __metadata("design:type", Number)
], GcListBoxComponent.prototype, "visibleItems", void 0);

class GcMaskComponent extends GcInputManBaseComponent {
    constructor(ref) {
        super(ref);
        this.onTextChanging = new EventEmitter();
        this.onSpinDown = new EventEmitter();
        this.onSpinUp = new EventEmitter();
        this.onValueChanged = new EventEmitter();
        // For V2.1 Compatibility, we have to keep this property or event
        this.valueChange = new EventEmitter();
        this.valueIsFullChange = new EventEmitter();
    }
    initGcComponent() {
        var input = document.createElement('input');
        this.createTemplate(input);
        this._imCtrl = new GC.InputMan.GcMask(input, this);
    }
    bindEvent() {
        if (!this._imCtrl) {
            return;
        }
        super.bindEvent();
        this._imCtrl.onInput((s) => {
            this.valueIsFullChange.emit(s.getValueIsFull());
        });
        this._imCtrl.onTextChanging((s, e) => { this.onTextChanging.emit({ sender: s, eArgs: e }); });
        this._imCtrl.onSpinDown((s) => { this.onSpinDown.emit(s); });
        this._imCtrl.onSpinUp((s) => { this.onSpinUp.emit(s); });
        this._imCtrl.onValueChanged((s) => {
            this.onValueChanged.emit(s);
            this.valueChange.emit(s.getValue());
            this.valueIsFullChange.emit(s.getValueIsFull());
        });
    }
}
GcMaskComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-mask',
                template: ``,
                providers: [provideParent(GcMaskComponent)]
            },] }
];
GcMaskComponent.ctorParameters = () => [
    { type: ElementRef }
];
GcMaskComponent.propDecorators = {
    autoConvert: [{ type: Input }],
    value: [{ type: Input }],
    formatPattern: [{ type: Input }],
    promptChar: [{ type: Input }],
    showLiterals: [{ type: Input }],
    tabAction: [{ type: Input }],
    text: [{ type: Input }],
    clipContent: [{ type: Input }],
    allowSpin: [{ type: Input }],
    spinWheel: [{ type: Input }],
    spinOnKeys: [{ type: Input }],
    exitOnLastChar: [{ type: Input }],
    highlightText: [{ type: Input }],
    showSpinButton: [{ type: Input }],
    spinButtonPosition: [{ type: Input }],
    valueIsFull: [{ type: Input }],
    showClearButton: [{ type: Input }],
    onTextChanging: [{ type: Output }],
    onSpinDown: [{ type: Output }],
    onSpinUp: [{ type: Output }],
    onValueChanged: [{ type: Output }],
    valueChange: [{ type: Output }],
    valueIsFullChange: [{ type: Output }]
};

class GcMultilineTextBoxComponent extends GcInputManBaseComponent {
    constructor(ref) {
        super(ref);
        this.onEditStatusChanged = new EventEmitter();
        this.onInvalidInput = new EventEmitter();
        this.onKeyExit = new EventEmitter();
        this.onTextChanged = new EventEmitter();
        this.onIMEReadingStringOutput = new EventEmitter();
        this.textChange = new EventEmitter();
        // For V2.1 Compatibility, we have to keep this property or event
        this.imeReadingStringChange = new EventEmitter();
    }
    initGcComponent() {
        var element = document.createElement('textarea');
        this.createTemplate(element);
        this["IMEReadingStringAppend"] = this.imeReadingStringAppend;
        this["IMEReadingStringKanaMode"] = this.imeReadingStringKanaMode;
        this["IMEReadingStringEnableAlphabetReadingMapping"] = this.imeReadingStringEnableAlphabetReadingMapping;
        this._imCtrl = new GC.InputMan.GcMultiLineTextBox(element, this);
    }
    bindEvent() {
        if (!this._imCtrl) {
            return;
        }
        super.bindEvent();
        this._imCtrl.onIMEReadingStringOutput((eventArgs) => {
            this.onIMEReadingStringOutput.emit(eventArgs);
            this.imeReadingStringChange.emit(this._imCtrl.getIMEReadingString());
        });
    }
}
GcMultilineTextBoxComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-multiline-textbox',
                template: ``,
                providers: [provideParent(GcMultilineTextBoxComponent)]
            },] }
];
GcMultilineTextBoxComponent.ctorParameters = () => [
    { type: ElementRef }
];
GcMultilineTextBoxComponent.propDecorators = {
    countWrappedLine: [{ type: Input }],
    lengthAsByte: [{ type: Input }],
    maxLength: [{ type: Input }],
    maxLineCount: [{ type: Input }],
    imeReadingStringKanaMode: [{ type: Input }],
    scrollBarMode: [{ type: Input }],
    scrollBars: [{ type: Input }],
    wordWrap: [{ type: Input }],
    acceptsReturn: [{ type: Input }],
    acceptsTab: [{ type: Input }],
    imeReadingStringAppend: [{ type: Input }],
    height: [{ type: Input }],
    imeReadingStringEnableAlphabetReadingMapping: [{ type: Input }],
    exitOnLastChar: [{ type: Input }],
    highlightText: [{ type: Input }],
    format: [{ type: Input }],
    autoConvert: [{ type: Input }],
    text: [{ type: Input }],
    lines: [{ type: Input }],
    imeReadingString: [{ type: Input }],
    cursorPosition: [{ type: Input }],
    onEditStatusChanged: [{ type: Output }],
    onInvalidInput: [{ type: Output }],
    onKeyExit: [{ type: Output }],
    onTextChanged: [{ type: Output }],
    onIMEReadingStringOutput: [{ type: Output }],
    textChange: [{ type: Output }],
    imeReadingStringChange: [{ type: Output }]
};
__decorate([
    Property("setIMEReadingStringKanaMode"),
    __metadata("design:type", String)
], GcMultilineTextBoxComponent.prototype, "imeReadingStringKanaMode", void 0);
__decorate([
    Property("setIMEReadingStringAppend"),
    __metadata("design:type", Boolean)
], GcMultilineTextBoxComponent.prototype, "imeReadingStringAppend", void 0);
__decorate([
    Property("setIMEReadingStringEnableAlphabetReadingMapping"),
    __metadata("design:type", Boolean)
], GcMultilineTextBoxComponent.prototype, "imeReadingStringEnableAlphabetReadingMapping", void 0);

class GcNumberComponent extends GcInputManBaseComponent {
    constructor(ref) {
        super(ref);
        this.onSpinDown = new EventEmitter();
        this.onSpinUp = new EventEmitter();
        this.onInvalidRange = new EventEmitter();
        this.onValueChanged = new EventEmitter();
        // For V2.1 Compatibility, we have to keep this property or event
        this.valueChange = new EventEmitter();
    }
    initGcComponent() {
        var element = document.createElement('input');
        this.createTemplate(element);
        this._imCtrl = new GC.InputMan.GcNumber(element, this);
    }
    bindEvent() {
        if (!this._imCtrl) {
            return;
        }
        super.bindEvent();
        this._imCtrl.onSpinDown((s) => { this.onSpinDown.emit(s); });
        this._imCtrl.onSpinUp((s) => { this.onSpinUp.emit(s); });
        this._imCtrl.onInvalidRange((s) => { this.onInvalidRange.emit(s); });
        this._imCtrl.onValueChanged((s) => {
            this.onValueChanged.emit(s);
            this.valueChange.emit(s.getValue());
        });
    }
    clear() {
        return this._imCtrl.clear();
    }
}
GcNumberComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-number',
                template: ``,
                providers: [provideParent(GcNumberComponent)]
            },] }
];
GcNumberComponent.ctorParameters = () => [
    { type: ElementRef }
];
GcNumberComponent.propDecorators = {
    value: [{ type: Input }],
    text: [{ type: Input }],
    watermarkZeroText: [{ type: Input }],
    formatDigit: [{ type: Input }],
    displayPositivePrefix: [{ type: Input }],
    displayFormatDigit: [{ type: Input }],
    maxValue: [{ type: Input }],
    minValue: [{ type: Input }],
    positivePrefix: [{ type: Input }],
    positiveSuffix: [{ type: Input }],
    displayPositiveSuffix: [{ type: Input }],
    negativePrefix: [{ type: Input }],
    displayNegativePrefix: [{ type: Input }],
    negativeSuffix: [{ type: Input }],
    displayNegativeSuffix: [{ type: Input }],
    valueSign: [{ type: Input }],
    adjustValueOnFocus: [{ type: Input }],
    allowDeleteToNull: [{ type: Input }],
    clearAction: [{ type: Input }],
    acceptsDecimal: [{ type: Input }],
    currencySymbol: [{ type: Input }],
    decimalPoint: [{ type: Input }],
    maxMinBehavior: [{ type: Input }],
    separator: [{ type: Input }],
    watermarkDisplayZeroText: [{ type: Input }],
    highlightText: [{ type: Input }],
    allowSpin: [{ type: Input }],
    spinWheel: [{ type: Input }],
    spinOnKeys: [{ type: Input }],
    spinWrap: [{ type: Input }],
    showSpinButton: [{ type: Input }],
    spinButtonPosition: [{ type: Input }],
    showNumericPad: [{ type: Input }],
    numericPadPosition: [{ type: Input }],
    showCalculatorAsDropDown: [{ type: Input }],
    showCalculatorAsPopup: [{ type: Input }],
    calculatorButtonPosition: [{ type: Input }],
    dropDownConfig: [{ type: Input }],
    showClearButton: [{ type: Input }],
    onSpinDown: [{ type: Output }],
    onSpinUp: [{ type: Output }],
    onInvalidRange: [{ type: Output }],
    onValueChanged: [{ type: Output }],
    valueChange: [{ type: Output }]
};

class GcShortcutInfoComponent {
}
GcShortcutInfoComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-shortcut-info',
                template: ``
            },] }
];
GcShortcutInfoComponent.propDecorators = {
    shortcutKey: [{ type: Input }],
    shift: [{ type: Input }],
    ctrl: [{ type: Input }],
    alt: [{ type: Input }],
    target: [{ type: Input }],
    action: [{ type: Input }],
    blockMessage: [{ type: Input }]
};
class GcShortcutComponent extends GcComponents {
    constructor(ref) {
        super(ref);
        this.preAction = new EventEmitter();
        this.postAction = new EventEmitter();
    }
    initGcComponent() {
        this._imCtrl = new GC.InputMan.GcShortcut();
        if (this.shortcutInfoList && this.shortcutInfoList.length > 0) {
            this.shortcutInfoList.forEach((info) => {
                let target = this.tryGetTarget(info.target);
                if (target === null)
                    return;
                let args = Object.assign(Object.assign({}, info), { key: info.shortcutKey, target });
                this._imCtrl.add(args);
            });
        }
    }
    tryGetTarget(str) {
        if (str === undefined)
            return window;
        let el = document.querySelector(str);
        if (el === null)
            return null;
        return el[IMCtrl] || el;
    }
    bindEvent() {
        if (!this._imCtrl)
            return;
        this._imCtrl.addEventListener(GC.InputMan.GcShortcutEvent.PreAction, (s, e) => {
            this.preAction.emit(e);
        });
        this._imCtrl.addEventListener(GC.InputMan.GcShortcutEvent.PostAction, (s, e) => {
            this.postAction.emit(e);
        });
    }
}
GcShortcutComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-shortcut',
                template: ``
            },] }
];
GcShortcutComponent.ctorParameters = () => [
    { type: ElementRef }
];
GcShortcutComponent.propDecorators = {
    blockMessage: [{ type: Input }],
    triggerAll: [{ type: Input }],
    shortcutInfoList: [{ type: ContentChildren, args: [GcShortcutInfoComponent,] }],
    preAction: [{ type: Output }],
    postAction: [{ type: Output }]
};

class GcSoftKeyboardComponent extends GcComponents {
    constructor(ref) {
        super(ref);
        this.onOpen = new EventEmitter();
        this.onClose = new EventEmitter();
        this.onKeyPress = new EventEmitter();
    }
    initGcComponent() {
        var element = document.createElement('div');
        this.createTemplate(element);
        this._imCtrl = new GC.InputMan.GcSoftKeyboard(element, this);
        this.setTarget(this.target);
        this.setOpened(this.opened);
    }
    bindEvent() {
        if (!this._imCtrl)
            return;
        this._imCtrl.addEventListener(GC.InputMan.GcSoftKeyboardEvent.KeyPress, (s, e) => {
            this.onKeyPress.emit(e);
        });
        this._imCtrl.addEventListener(GC.InputMan.GcSoftKeyboardEvent.Open, (s) => {
            this.onOpen.emit(s);
        });
        this._imCtrl.addEventListener(GC.InputMan.GcSoftKeyboardEvent.Close, (s) => {
            this.onClose.emit(s);
        });
    }
    setValueToIMControl(key, value) {
        if (key === 'opened') {
            this.setOpened(value);
        }
        else if (key === 'target') {
            this.setTarget(value);
        }
        else {
            super.setValueToIMControl(key, value);
        }
    }
    setOpened(opened) {
        if (opened === true) {
            this._imCtrl.open();
        }
        else if (opened === false) {
            this._imCtrl.close();
        }
    }
    setTarget(str) {
        let target = this.tryGetTarget(str);
        if (target) {
            this._imCtrl.target = target;
        }
    }
    tryGetTarget(str) {
        try {
            let target = document.querySelector(str);
            return target;
        }
        catch (_a) { }
        return null;
    }
}
GcSoftKeyboardComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-soft-keyboard',
                template: ``
            },] }
];
GcSoftKeyboardComponent.ctorParameters = () => [
    { type: ElementRef }
];
GcSoftKeyboardComponent.propDecorators = {
    opened: [{ type: Input }],
    randomDisplay: [{ type: Input }],
    hideButtonWhenMouseOn: [{ type: Input }],
    displayType: [{ type: Input }],
    showSecurityOptions: [{ type: Input }],
    securityLevel: [{ type: Input }],
    qwertyLayout: [{ type: Input }],
    showCapitalLetters: [{ type: Input }],
    target: [{ type: Input }],
    enabled: [{ type: Input }],
    onOpen: [{ type: Output }],
    onClose: [{ type: Output }],
    onKeyPress: [{ type: Output }]
};

class GcTextBoxComponent extends GcInputManBaseComponent {
    constructor(ref) {
        super(ref);
        this.onIMEReadingStringOutput = new EventEmitter();
        // For V2.1 Compatibility, we have to keep this property or event
        this.imeReadingStringChange = new EventEmitter();
    }
    initGcComponent() {
        var input = document.createElement("input");
        this.createTemplate(input);
        this["IMEReadingStringAppend"] = this.imeReadingStringAppend;
        this["IMEReadingStringKanaMode"] = this.imeReadingStringKanaMode;
        this["IMEReadingStringEnableAlphabetReadingMapping"] = this.imeReadingStringEnableAlphabetReadingMapping;
        this._imCtrl = new GC.InputMan.GcTextBox(input, this);
    }
    bindEvent() {
        if (!this._imCtrl) {
            return;
        }
        super.bindEvent();
        this._imCtrl.onIMEReadingStringOutput((s, e) => {
            this.onIMEReadingStringOutput.emit({ sender: s, eArgs: e });
            this.imeReadingStringChange.emit(s.getIMEReadingString());
        });
    }
}
GcTextBoxComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-text-box',
                template: ``,
                providers: [provideParent(GcTextBoxComponent)]
            },] }
];
GcTextBoxComponent.ctorParameters = () => [
    { type: ElementRef }
];
GcTextBoxComponent.propDecorators = {
    autoConvert: [{ type: Input }],
    ellipsisString: [{ type: Input }],
    ellipsis: [{ type: Input }],
    format: [{ type: Input }],
    lengthAsByte: [{ type: Input }],
    maxLength: [{ type: Input }],
    passwordChar: [{ type: Input }],
    text: [{ type: Input }],
    useSystemPasswordChar: [{ type: Input }],
    imeReadingStringAppend: [{ type: Input }],
    imeReadingStringKanaMode: [{ type: Input }],
    imeReadingStringEnableAlphabetReadingMapping: [{ type: Input }],
    exitOnLastChar: [{ type: Input }],
    highlightText: [{ type: Input }],
    imeReadingString: [{ type: Input }],
    showClearButton: [{ type: Input }],
    passwordRevelationMode: [{ type: Input }],
    onIMEReadingStringOutput: [{ type: Output }],
    imeReadingStringChange: [{ type: Output }]
};
__decorate([
    Property("setIMEReadingStringAppend"),
    __metadata("design:type", Boolean)
], GcTextBoxComponent.prototype, "imeReadingStringAppend", void 0);
__decorate([
    Property("setIMEReadingStringKanaMode"),
    __metadata("design:type", String)
], GcTextBoxComponent.prototype, "imeReadingStringKanaMode", void 0);
__decorate([
    Property("setIMEReadingStringEnableAlphabetReadingMapping"),
    __metadata("design:type", Boolean)
], GcTextBoxComponent.prototype, "imeReadingStringEnableAlphabetReadingMapping", void 0);

class GcTipNotifierComponent extends GcComponents {
    constructor(ref, _parent) {
        super(ref);
        this._parent = _parent;
    }
    ngAfterViewInit() {
        if (!this._parent || typeof this._parent !== 'object' ||
            typeof this._parent.getNestedIMControl() !== 'object') {
            return;
        }
        this._imCtrl = new GC.InputMan.GcTipNotifier(this._parent.getNestedIMControl(), this);
        this.onInitialized.emit(this._imCtrl);
    }
    bindEvent() { }
    initGcComponent() {
    }
    onFail() {
        return this._imCtrl.onFail();
    }
    onSuccess() {
        return this._imCtrl.onSuccess();
    }
    onClear() {
        return this._imCtrl.onClear();
    }
}
GcTipNotifierComponent.decorators = [
    { type: Component, args: [{
                selector: 'gc-tip-notifier',
                template: ``
            },] }
];
GcTipNotifierComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: GcComponents, decorators: [{ type: Optional }] }
];
GcTipNotifierComponent.propDecorators = {
    direction: [{ type: Input }],
    position: [{ type: Input }],
    directionPriority: [{ type: Input }],
    template: [{ type: Input }],
    failMessage: [{ type: Input }],
    successMessage: [{ type: Input }],
    tipState: [{ type: Input }]
};

/*Import from angular */
/**
 *  Declare & add export modules
 */
class InputManModule {
}
InputManModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FormsModule
                ],
                declarations: [
                    GcMaskComponent,
                    GcTextBoxComponent,
                    GcCalendarComponent,
                    GcNumberComponent,
                    GcDateTimeComponent,
                    GcListBoxComponent,
                    GcComboBoxComponent,
                    GcDateTimePickerComponent,
                    GcTipNotifierComponent,
                    GcIconNotifierComponent,
                    GcMultilineTextBoxComponent,
                    GcComboBoxColumnComponent,
                    GcListBoxColumnComponent,
                    GcFormPersistenceComponent,
                    GcCalculatorComponent,
                    GcFunctionKeyComponent,
                    GcFunctionKeyInfoComponent,
                    GcShortcutComponent,
                    GcShortcutInfoComponent,
                    GcSoftKeyboardComponent,
                ],
                exports: [
                    GcMaskComponent,
                    GcTextBoxComponent,
                    GcCalendarComponent,
                    GcNumberComponent,
                    GcDateTimeComponent,
                    GcListBoxComponent,
                    GcComboBoxComponent,
                    GcDateTimePickerComponent,
                    GcTipNotifierComponent,
                    GcIconNotifierComponent,
                    GcMultilineTextBoxComponent,
                    GcComboBoxColumnComponent,
                    GcListBoxColumnComponent,
                    GcFormPersistenceComponent,
                    GcCalculatorComponent,
                    GcFunctionKeyComponent,
                    GcFunctionKeyInfoComponent,
                    GcShortcutComponent,
                    GcShortcutInfoComponent,
                    GcSoftKeyboardComponent
                ]
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { GcCalculatorComponent, GcCalendarComponent, GcComboBoxColumnComponent, GcComboBoxComponent, GcDateTimeComponent, GcDateTimePickerComponent, GcFormPersistenceComponent, GcFunctionKeyComponent, GcFunctionKeyInfoComponent, GcIconNotifierComponent, GcListBoxColumnComponent, GcListBoxComponent, GcMaskComponent, GcMultilineTextBoxComponent, GcNumberComponent, GcShortcutComponent, GcShortcutInfoComponent, GcSoftKeyboardComponent, GcTextBoxComponent, GcTipNotifierComponent, InputManModule, provideParent as ɵa, GcComponents as ɵb, Property as ɵc, GcInputManBaseComponent as ɵd };
//# sourceMappingURL=grapecity-inputman.angular.js.map
